

//	(C) 2017 R-MIK Sawomir Skrzyski
//	slawomir.skrzynski@ep.com.pl


#include "onewire_DS18b20.h"				// 1-Wire DS18b20


//======================================================================//
//	Mierzy temperatur i i zapamituje w RAM CPU						//
//																		//
// Parametr "rozdzielczo" jest istotny w trybie zasilania pasywnego	//
//	bo od niej zaley czas oczekiwania na wynik konwersji				//
//----------------------------------------------------------------------//
double ds18b20_gettemp( byte skip, byte *match, byte rozdzielczosc )
{
byte i, powerDs, czas=80;
double retd = 0x8000;


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif


	onewriteOverdriveOnOff( FALSE );
	if ( onewireResetSTD() ) 										//reset
		{
		if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); } //skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_RPWRSUPPLY );				// Sprawdzenie rda zasilania
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali
		powerDs = ds18b20cfg[DS18B20_CFGPOWER] = onewireReadByte();
/*
Read Power Supply [B4h]
With every read data time slot issued after this command has been sent to the DS18B20, the device will
signal its power mode: 0=parasite power, 1=external power supply provided.
DS18B2
*/

		onewireResetSTD(); 										//reset
		if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }	//skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_CONVERTTEMP ); 			//start temperature conversion
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali
/*
Convert T [44h]
This command begins a temperature conversion. No further data is required. The temperature
conversion will be performed and then the DS18B20 will remain idle. If the bus master issues read time
slots following this command, the DS18B20 will output 0 on the bus as long as it is busy making a
temperature conversion; it will return a 1 when the temperature conversion is complete. If parasitepowered, the bus master has to enable a strong pullup for a period greater than tconv immediately after
issuing this command.
*/
		if ( !(powerDs & 1) )			// Jesli zasilanie pasozytnicze dac silne podciganie
			{							// Czas oczekiwania zaley od rozdzielczoci
			switch( rozdzielczosc )
				{
				case( DS18B20_RESOLUTION_9 ):  czas = 10; break;
				case( DS18B20_RESOLUTION_10 ): czas = 20; break;
				case( DS18B20_RESOLUTION_11 ): czas = 40; break;
				default: czas = 80;
				}
				//  9 bit 93.75 ms (tconv/8)
				// 10 bit 187.5 ms (tconv/4)
				// 11 bit 375 ms (tconv/2)
				// 12 bit 750 ms (tconv)
			onewireStrongPullup( TRUE );
			for(i=0; i<czas; i++) { wdt_reset(); _delay_ms(10); }
			onewireStrongPullup( FALSE );
			}
		i=80; while( !onewireReadBit() )		 				// wait until conversion is complete
			{													// timeout 800ms
			wdt_reset();
			_delay_ms(10);
			if ( !(--i) ) break;
			}
		i=3; while( !(retd=ds18b20_configR( skip, match, ds18b20cfg )) ){ i--; if ( !i) break; }	// odczyt z DS do ram
		}


	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif

	//convert the 12 bit value obtained
	retd = ( ( ds18b20cfg[DS18B20_TEMPH] << 8 ) + ds18b20cfg[DS18B20_TEMPL] ) * 0.0625;


	onewireResetSTD();
	return retd;
}


//----------------------------------------------------------------------//
//	Odczytuje pami podrczn DS18b20 do RAM CPU						//
//----------------------------------------------------------------------//
byte ds18b20_configR( byte skip, byte *match, byte *dst )
{
byte crc, i, dstr=FALSE, *cel;


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif


	if ( onewireResetSTD() ) 										//reset
		{
		if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }	//skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_readSCRATCHPAD ); 		//read scratchpad
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali
/*
Read Scratchpad [BEh]
This command reads the contents of the scratchpad. Reading will commence at byte 0 and will continue
through the scratchpad until the ninth (byte 8, CRC) byte is read. If not all locations are to be read, the
master may issue a reset to terminate reading at any time.
*/
		cel = dst; for(i=0; i<9; i++){ *cel++ = onewireReadByte();	}	// Czytaj dane
		cel = dst; crc=0; for(i=0; i<9; i++){ crc = onewireCrc8update( crc, *cel++ ); }	// Licz CRC
		if ( !crc ) dstr = TRUE;		// CRC musi by zerem
		}


	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif


	onewireResetSTD();
	return( dstr );
}


//----------------------------------------------------------------------//
//	Zapamituje konfiguracj w pamici podrcznej DS18b20				//
//----------------------------------------------------------------------//
byte ds18b20_configW( byte skip, byte *match, byte th, byte tl, byte cfg )
{
byte retb=FALSE;


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif

	onewriteOverdriveOnOff( FALSE );
	if ( onewireResetSTD() ) 										//reset
		{
		if ( skip ){ onewireWriteByte(ONEWIRE_CMD_SKIPROM); }	//skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_writeSCRATCHPAD );
		onewireWriteByte( th );
		onewireWriteByte( tl );
		onewireWriteByte( cfg );
/*
Write Scratchpad [4Eh]
This command writes to the scratchpad of the DS18B20, starting at the TH register. The next 3 bytes
written will be saved in scratchpad memory at address locations 2 through 4. All 3 bytes must be written
before a reset is issued.
*/


		if ( ds18b20_configR( skip, match, ds18b20cfg ) )
			{
			if ( ds18b20cfg[DS18B20_CFGTH] == th &&
				 ds18b20cfg[DS18B20_CFGTL] == tl &&
				 ds18b20cfg[DS18B20_CFGRESOLUTION] == cfg ) retb = TRUE;
			}
		}


	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif


	onewireResetSTD();
	return retb;
}


//----------------------------------------------------------------------//
//	Zapamituje konfig w pamici podrcznej DS18b20 i eeprom			//
//----------------------------------------------------------------------//
byte ds18b20_Save( byte skip, byte *match, byte th, byte tl, byte cfg )
{
	ds18b20_configW( skip, match, th, tl, cfg );
	return( ds18b20_Copy( skip, match ) );
}


//----------------------------------------------------------------------//
//	Kopiuje dane z pamici podrcznej do eeprom DS18b20					//
//----------------------------------------------------------------------//
byte ds18b20_Copy( byte skip, byte *match )
{
byte powerDs, i, retd=FALSE;
byte mem[DS18B20_CFGLEN];


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif


	ds18b20_configR( skip, match, mem );		// odczyt konfiguracji, zapamietanie th, tl, cfg


	onewriteOverdriveOnOff( FALSE );
	if ( onewireResetSTD() ) 										//reset
		{
		if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); } //skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_RPWRSUPPLY );				// Sprawdzenie rda zasilania
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali
		powerDs = ds18b20cfg[DS18B20_CFGPOWER] = onewireReadByte();


		onewireResetSTD();
		if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); } //skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_copySCRATCHPAD );
/*
Copy Scratchpad [48h]
This command copies the scratchpad into the E2 memory of the DS18B20, storing the temperature trigger
bytes in nonvolatile memory. If the bus master issues read time slots following this command, the
DS18B20 will output 0 on the bus as long as it is busy copying the scratchpad to E2; it will return a 1
when the copy process is complete. If parasite-powered, the bus master has to enable a strong pullup for
at least 10 ms immediately after issuing this command. The DS18B20 EEPROM is rated for a minimum
of 50,000 writes and 10 years data retention at T=+55C.
*/

		if ( !(powerDs & 1) )		//jesli zasilanie pasozytnicze dac silne podciganie
			{
			onewireStrongPullup( TRUE );
			wdt_reset();
			_delay_ms(10);
			onewireStrongPullup( FALSE );
			}
		i=10; while( !onewireReadBit() )		 				// wait until conversion is complete
			{													// timeout 800ms
			wdt_reset();
			_delay_ms(1);
			if ( !(--i) ) break;
			}
		}

nop();
//todo:
// odczyt konfiguracji i porownanie th, tl, cfg


	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif


	onewireResetSTD();
	return( retd );
}


//----------------------------------------------------------------------//
//	Odczytuje dane z eeprom do pamici podrcznej DS18b20				//
//	i zapamituje w RAM	CPU												//
//----------------------------------------------------------------------//
byte ds18b20_Recall( byte skip, byte *match, byte *dst )
{
byte retd=FALSE, i;


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif


	onewriteOverdriveOnOff( FALSE );
	if ( onewireResetSTD() ) 									//reset
		{
		if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); } //skip ROM
		else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		onewireWriteByte( ONEWIRE_18b20_recallEEPROM );
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali
/*
Recall E2 [B8h]
This command recalls the temperature trigger values and configuration register stored in E2 to the
scratchpad. This recall operation happens automatically upon power-up to the DS18B20 as well, so valid
data is available in the scratchpad as soon as the device has power applied. With every read data time slot
issued after this command has been sent, the device will output its temperature converter busy flag:
0=busy, 1=ready.

*/
		i=10; while( !onewireReadBit() )		 				// wait until ready
			{													// timeout 10ms
			wdt_reset();
			_delay_ms(1);
			if ( !(--i) ) break;
			}


		i=3; while( !(retd=ds18b20_configR( skip, match, dst )) ){ i--; if ( !i) break; }	// odczyt z DS do ram
		}


	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif


	onewireResetSTD();
	return( retd );

}


//----------------------------------------------------------------------//


