

#define	IIC_ADR_2482wr	0b00110000		// Bit0-R/W, bit1-A0, bit2-A1
#define	IIC_ADR_2482rd	IIC_ADR_2482wr+1


byte DS2482_OWSearch();


//======================================================================//
byte DS2482_ver;
byte DS2482_Init();
byte DS2482_Cfg( byte config );
byte DS2482_Wait_Busy( byte stop, word timeout );
void DS2482_OverdriveOnOff( byte ovd );
byte DS2482loopBack;
byte DS2482_SetReadPointer( byte pointer );


byte DS2482_rdSatus();
byte DS2482status=-1;
// Odczyt statusu str. 8
// bit0 (1WB) 	1 magistrala zajta, 0-koniec operacji
// bit1 (PPD) 	1 wykryto presence
// bit2 (SD)
// bit3 (LL)	fizyczny stan magistrali 1-Wire
// bit4 (RST)	1 - DS2482 po resecie
// bit5 (SBR)	wynik odczytu pojedyczego bitu
// bit6 (TSB)	wynik odczytu operacji 3-bit (search rom)
// bit7 (DIR)	kierunek wyszukiwania dla operacji 3-bit (warto trzeciego bitu))


byte DS2482_rst=-1;
byte DS2482_OW_Reset();
byte DS2482_OW_WRbit( byte bit );
byte DS2482_OW_RDbit();
byte DS2482_OW_WRtri( byte dir );
byte DS2482_OW_WRbyte( byte bajt );
byte DS2482_OW_LastByte;
byte DS2482_OW_RDbyte();

#ifndef OW_SEARCH_ROM_NO
  unsigned char OW_SEARCH_ROM_NO[8];
#else
  #error "stala?"
#endif


//======================================================================//
byte DS2482_OW_WRbit( byte bit )
{
byte st;


	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }
	// $20 - nie byo ACK
	// $38 - przegrany arbritarz

	// 1Command 1-Wire Single Bit, 87h
	st = TWI_Write( 0x87 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }


	if ( bit ) bit = 0x80;
	st = TWI_Write( bit ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }


//Nie ma co czeka, nawet przy 400kHz bit na pewno ju zosta wygenerowany
// tym bardziej, e dzieje sie to po odczycie pierwszego bitu danej do wysania (najstarszy bit w IIC jest pnadawany jako pierwszy)
//	//----- Wait -----//
//	DS2482_Wait_Busy( false, 100*8 ); if ( TWI_Status != 0x58  ){ TWI_Exit(); return( TWI_Status ); }	// Czekaj na koniec operacji


	//----- STOP -----//
	st = TWI_Stop();


return( 0 );
}


//----------------------------------------------------------------------//
byte DS2482_OW_RDbit()
{
byte st;


	DS2482_OW_WRbit( 1 );

	st = DS2482_rdSatus();


	// bit5 (SBR)	wynik odczytu pojedyczego bitu
	// bit6 (TSB)	wynik odczytu operacji 3-bit (search rom)
	// bit7 (DIR)	kierunek wyszukiwania dla operacji 3-bit (warto trzeciego bitu))
	return( st & 0x20 );
}


//----------------------------------------------------------------------//
byte DS2482_OW_WRtri( byte dir )
{
byte st;


	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }
	// $20 - nie byo ACK
	// $38 - przegrany arbritarz

	// 1WT Command 1-Wire Triplet, 78h
	st = TWI_Write( 0x78 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }

	st = TWI_Write( dir ? 0x80 : 0x00 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }

	st = DS2482_rdSatus();


	return( st );
}


//----------------------------------------------------------------------//
byte DS2482_OW_RDbyte()
{
byte st, dana;


	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }

	// 1WRB Command 1-Wire Read Byte, 96h
	st = TWI_Write( 0x96 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }


	//----- Wait -----//
//Nawet w overdrive przy IIC 400kHz nie trzeba czeka
	if ( !onewireOverdrive ){ DS2482_Wait_Busy( false, 100*8 ); if ( TWI_Status != 0x58  ){ TWI_Exit(); return( TWI_Status ); }	}

	DS2482_SetReadPointer( 0xE1 );

	//----- START Powtrz -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Odczyt z DS2482
	st = TWI_Write( IIC_ADR_2482rd ); if ( st != 0x40 ){ TWI_Exit(); return( st ); }

	// Odczytaj ostatni bajt
	dana = TWI_Read( false ); if ( TWI_Status != 0x58  ){ TWI_Exit(); return( st ); }


	//----- STOP -----//
	st = TWI_Stop();


	return( dana );
}


//----------------------------------------------------------------------//
byte DS2482_SetReadPointer( byte pointer )
{
//byte static DS2482_ReadPointer=0;
byte st;


// Niestety ponisze nie wypali. Wszelkie operacje ustawiaj wskanik na rejestr statufu (0xF0)
// wic przed odczytem danej trzeba zrobi ustawienie wskanika na rejestr danych (0xE1)
//	if ( DS2482_ReadPointer != pointer )
//		{
//		DS2482_ReadPointer = pointer;

		//----- START Powtrz -----//
		st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

		// Zapis do DS2482
		st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }

		// SRP Command Set Read Pointer, E1h
		st = TWI_Write( 0xE1 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }

		// Wskanik odczytu
		st = TWI_Write( pointer ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }
//	}

}


//----------------------------------------------------------------------//
byte DS2482_OW_WRbyte( byte bajt )
{
byte st;


	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }
	// $20 - nie byo ACK
	// $38 - przegrany arbritarz

	// 1WWB Command 1-Wire Write Byte, A5h
	st = TWI_Write( 0xA5 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }


	st = TWI_Write( bajt ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }
	DS2482_OW_LastByte = bajt;


	//----- Wait -----//
//Przy IIC 400kHz musi by czekanie
	/*if ( !onewireOverdrive )*/{ DS2482_Wait_Busy( false, 100*8 ); if ( TWI_Status != 0x58  ){ TWI_Exit(); return( TWI_Status ); } }

	//----- STOP -----//
	st = TWI_Stop();


return( 0 );
}


//----------------------------------------------------------------------//
// "stop" = true to generowany jest warunek stopu
// "timeout" max czas oczekiwania w mikrosekundach
//----------------------------------------------------------------------//
byte DS2482_Wait_Busy( byte stop, word timeout )
{
byte st, dana;



	//----- START (Powtrz) -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Odczyt z DS2482
	st = TWI_Write( IIC_ADR_2482rd ); if ( st != 0x40 ){ TWI_Exit(); return( st ); }

	timeout /= 100/25;		// Bajt w IIC 400kHz to 21us
	while( timeout-- )
		{
//		_delay_us(25);
		if ( !(dana=(TWI_Read(true)&1)) ) break;
		// Odczyt statusu str. 8
		// bit0 (1WB) 	1 magistrala zajta, 0-koniec operacji
		// bit1 (PPD) 	1 wykryto presence
		// bit2 (SD)
		// bit3 (LL)	fizyczny stan magistrali 1-Wire
		// bit4 (RST)	1 - DS2482 po resecie
		// bit5 (SBR)	wynik odczytu pojedyczego bitu
		// bit6 (TSB)	wynik odczytu operacji 3-bit (search rom)
		// bit7 (DIR)	kierunek wyszukiwania dla operacji 3-bit (warto trzeciego bitu))
		if ( TWI_Status != 0x50  ){ TWI_Exit(); return( TWI_Status ); }
		if ( TWI_Status == 0x58  ) break;
		}
	// Odczytaj ostatni bajt
	DS2482status = TWI_Read( false ); if ( TWI_Status != 0x58  ) TWI_Exit();


	if( stop ) TWI_Stop();


	return( DS2482status );
}


//----------------------------------------------------------------------//
// "pomin" pomija Set Read Pointer, przydatne dla komend operujcych na bitach
byte DS2482_rdSatus()
{
byte st;


	DS2482_SetReadPointer( 0xF0 );

	//----- START Powtrz -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Odczyt z DS2482
	st = TWI_Write( IIC_ADR_2482rd ); if ( st != 0x40 ){ TWI_Exit(); return( st ); }
	// Odczyt statusu str. 8
	// bit0 (1WB) 	1 magistrala zajta, 0-koniec operacji
	// bit1 (PPD) 	1 wykryto presence
	// bit2 (SD)
	// bit3 (LL)	fizyczny stan magistrali 1-Wire
	// bit4 (RST)	1 - DS2482 po resecie
	// bit5 (SBR)	wynik odczytu pojedyczego bitu
	// bit6 (TSB)	wynik odczytu operacji 3-bit (search rom)
	// bit7 (DIR)	kierunek wyszukiwania dla operacji 3-bit (warto trzeciego bitu))
	DS2482status = TWI_Read( false );	// Odczytaj jeden (ostatni) bajt
	if ( TWI_Status != 0x58  ){ TWI_Exit(); return( st ); }


	//----- STOP -----//
	st = TWI_Stop();


	return( DS2482status );
}


//----------------------------------------------------------------------//
byte DS2482_Cfg( byte config )
{
byte st;

/*
// Select the 1-Wire channel on a DS2482-800.
//
// Returns: TRUE if channel selected
//          FALSE device not detected or failure to perform select
//
int DS2482_channel_select(int channel)
{
   unsigned char ch, ch_read, check;

   // Channel Select (Case A)
   //   S AD,0 [A] CHSL [A] CC [A] Sr AD,1 [A] [RR] A\ P
   //  [] indicates from slave
   //  CC channel value
   //  RR channel read back

   I2C_start();
   I2C_write(I2C_address | I2C_WRITE, EXPECT_ACK);
   I2C_write(CMD_CHSL, EXPECT_ACK);

   switch (channel)
   {
      default: case 0: ch = 0xF0; ch_read = 0xB8; break;
      case 1: ch = 0xE1; ch_read = 0xB1; break;
      case 2: ch = 0xD2; ch_read = 0xAA; break;
      case 3: ch = 0xC3; ch_read = 0xA3; break;
      case 4: ch = 0xB4; ch_read = 0x9C; break;
      case 5: ch = 0xA5; ch_read = 0x95; break;
      case 6: ch = 0x96; ch_read = 0x8E; break;
      case 7: ch = 0x87; ch_read = 0x87; break;
   };

   I2C_write(ch, EXPECT_ACK);
   I2C_rep_start();
   I2C_write(I2C_address | I2C_READ, EXPECT_ACK);
   check = I2C_read(NACK);
   I2C_stop();

   // check for failure due to incorrect read back of channel
   return (check == ch_read);
}
*/

	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }
	// $20 - nie byo ACK
	// $38 - przegrany arbritarz

	//----- Konfiguracja DS2482 -----//
	st = TWI_Write( 0xD2 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }

	// Bajt konfiguracji
	// bit 0 - PullUp
	// bit 1 = 0
	// bit 2 - Strong PullUp
	// bit 3 - Speed
	// bit 4 - /PullUp
	// bit 5 = 1
	// bit 6 - /speed
	// bit 7 - /Strong PullUp
	config &= 0xF;
	st = TWI_Write( config | (~config << 4) ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }


	//----- START Powtrz -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Odczyt z DS2482
	st = TWI_Write( IIC_ADR_2482rd ); if ( st != 0x40 ){ TWI_Exit(); return( st ); }

	// Odczytaj jeden (ostatni) bajt
	DS2482status = TWI_Read( false ); if ( TWI_Status != 0x58  ){ TWI_Exit(); return( st ); }


	//----- STOP -----//
	st = TWI_Stop();


return( 0 );
}


//----------------------------------------------------------------------//
byte DS2482_Init()
{
byte liczba=0, st;


//	TWI_Master_Initialise();


//for(liczba=0; liczba<4; liczba++)
//{
	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); return( st ); }
	// $20 - nie byo ACK
	// $38 - przegrany arbritarz

	//----- Reset DS2482 -----//
	st = TWI_Write( 0xF0 ); if ( st != 0x28 ){ TWI_Exit(); return( st ); }


	//----- START Powtrz -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ) return( st );

	// Odczyt z DS2482
	st = TWI_Write( IIC_ADR_2482rd ); if ( st != 0x40 ){ TWI_Exit(); return( st ); }

	// Odczytaj jeden (ostatni) bajt
	DS2482status = TWI_Read( false ); if ( TWI_Status != 0x58  ){ TWI_Exit(); return( st ); }


	//----- STOP -----//
	st = TWI_Stop();
//}

	DS2482_ver = 1;


byte cfg=0;
	if ( onewirePullup ) cfg |= 1;
	if ( onewireStrong ) cfg |= 4;
	if ( onewireOverdrive ) cfg |= 8;
	DS2482_Cfg( cfg );


return( liczba );
}


//----------------------------------------------------------------------//
void DS2482_OverdriveOnOff( byte ovd )
{
byte cfg=0;



	// Zapobieganie petli lokalnej w overdrive dla Emulatora 1-Wire
	if ( DS2482loopBack )
	  	{
		if ( DS2482_OW_LastByte == ONEWIRE_OVD_SKIPROM ) DS2482_OW_LastByte = ONEWIRE_CMD_SKIPROM;
		if ( DS2482_OW_LastByte == ONEWIRE_OVD_MATCHROM ) DS2482_OW_LastByte = ONEWIRE_CMD_MATCHROM;
		DS2482_OW_Reset();
		DS2482_OW_WRbyte( DS2482_OW_LastByte );
		return;		// Wyjcie bez zmiany prdkoci
		}


	onewireOverdrive = ovd;


	if ( onewirePullup ) cfg |= 1;
	if ( onewireStrong ) cfg |= 4;
	if ( onewireOverdrive ) cfg |= 8;
	DS2482_Cfg( cfg );
}


//----------------------------------------------------------------------//
byte DS2482_OW_Reset()
{
byte st;


#warning "zmienic konstrukcje: st = xxx if (st !=yyyy) na: if ( (st=xxxx) != yyyy )"
	//----- START -----//
	st = TWI_Start(); if ( st != 8 && st != 0x10 ){ DS2482_rst=st; return( false ); }

	// Zapis do DS2482
	st = TWI_Write( IIC_ADR_2482wr );	if ( st != 0x18 ){ TWI_Exit(); DS2482_rst=st; return( false ); }
	// $20 - nie byo ACK
	// $38 - przegrany arbritarz

	// 1-W Reset
	st = TWI_Write( 0xB4 ); if ( st != 0x28 ){ TWI_Exit(); DS2482_rst=st; return( false ); }

	//----- STOP -----//
	st = TWI_Stop();

	//----- Wait -----//
word timeout=3800/50;		// jeden bajt po IIC 400kHz to 23us odpytanie to 2 bajty (adrRD i RDstatus)
	while( timeout-- )
		{
//		_delay_us( 50 );
		DS2482_Wait_Busy( false, 1000*2 ); if ( TWI_Status != 0x58  ){ TWI_Exit(); DS2482_rst=st; return( false ); }	// Czekaj na koniec operacji
		if ( DS2482status & 2 ) break;
		// Odczyt statusu str. 8
		// bit0 (1WB) 	1 magistrala zajta, 0-koniec operacji
		// bit1 (PPD) 	1 wykryto presence
		// bit2 (SD)
		// bit3 (LL)	fizyczny stan magistrali 1-Wire
		// bit4 (RST)	1 - DS2482 po resecie
		// bit5 (SBR)	wynik odczytu pojedyczego bitu
		// bit6 (TSB)	wynik odczytu operacji 3-bit (search rom)
		// bit7 (DIR)	kierunek wyszukiwania dla operacji 3-bit (warto trzeciego bitu))
		}

	//----- STOP -----//
	st = TWI_Stop();


DS2482_rst=st;
return( true );
}


//======================================================================//
// Search state
byte LastDiscrepancy;
byte LastFamilyDiscrepancy;
byte LastDeviceFlag;
byte crc8;


//--------------------------------------------------------------------------
// Find the 'first' devices on the 1-Wire network
// Return TRUE  : device found, ROM number in OW_SEARCH_ROM_NO buffer
//        FALSE : no device present
//
byte DS2482_OWFirst()
{
   // reset the search state
   LastDiscrepancy = 0;
   LastDeviceFlag = FALSE;
   LastFamilyDiscrepancy = 0;

   return DS2482_OWSearch();
}

//--------------------------------------------------------------------------
// Find the 'next' devices on the 1-Wire network
// Return TRUE  : device found, ROM number in OW_SEARCH_ROM_NO buffer
//        FALSE : device not found, end of search
//
byte DS2482_OWNext()
{
   // leave the search state alone
   return DS2482_OWSearch();
}

//--------------------------------------------------------------------------
// The 'OWSearch' function does a general search. This function
// continues from the previous search state. The search state
// can be reset by using the 'OWFirst' function.
// This function contains one parameter 'alarm_only'.
// When 'alarm_only' is TRUE (1) the find alarm command
// 0xEC is sent instead of the normal search command 0xF0.
// Using the find alarm command 0xEC will limit the search to only
// 1-Wire devices that are in an 'alarm' state.
//
// Returns:   TRUE (1) : when a 1-Wire device was found and its
//                       Serial Number placed in the global ROM
//            FALSE (0): when no new device was found.  Either the
//                       last search was the last device or there
//                       are no devices on the 1-Wire Net.
//
byte DS2482_OWSearch()
{
   byte id_bit_number;
   byte last_zero, rom_byte_number, search_result;
   byte id_bit, cmp_id_bit;
   byte rom_byte_mask, search_direction, status;

   // initialize for search
   id_bit_number = 1;
   last_zero = 0;
   rom_byte_number = 0;
   rom_byte_mask = 1;
   search_result = FALSE;
   crc8 = 0;

   // if the last call was not the last one
   if (!LastDeviceFlag)
   {
      // 1-Wire reset
      if ( !DS2482_OW_Reset() )
      {
         // reset the search
         LastDiscrepancy = 0;
         LastDeviceFlag = FALSE;
         LastFamilyDiscrepancy = 0;
         return FALSE;
      }

      // issue the search command
      DS2482_OW_WRbyte( ONEWIRE_CMD_SEARCHROM );

      // loop to do the search
      do
      {
         // if this discrepancy if before the Last Discrepancy
         // on a previous next then pick the same as last time
         if (id_bit_number < LastDiscrepancy)
         {
            if ((OW_SEARCH_ROM_NO[rom_byte_number] & rom_byte_mask) > 0)
               search_direction = 1;
            else
               search_direction = 0;
         }
         else
         {
            // if equal to last pick 1, if not then pick 0
            if (id_bit_number == LastDiscrepancy)
               search_direction = 1;
            else
               search_direction = 0;
         }

         // Perform a triple operation on the DS2482 which will perform
         // 2 read bits and 1 write bit
         DS2482_OW_WRtri(search_direction);
		status = DS2482status;
		// bit5 (SBR)	wynik odczytu pojedyczego bitu
		// bit6 (TSB)	wynik odczytu operacji 3-bit (search rom)
		// bit7 (DIR)	kierunek wyszukiwania dla operacji 3-bit (warto trzeciego bitu))
         // check bit results in status byte
//         id_bit = ((status & STATUS_SBR) == STATUS_SBR);
//         cmp_id_bit = ((status & STATUS_TSB) == STATUS_TSB);
//         search_direction =
//         	((status & STATUS_DIR) == STATUS_DIR) ? (byte)1 : (byte)0;
        // check bit results in status byte
         id_bit = ((status & 0x20) == 0x20);				// id_bit = odczyt bitu z magistrali
         cmp_id_bit = ((status & 0x40) == 0x40);			// cmp_id_bit =
         search_direction =
         	((status & 0x80) == 0x80) ? (byte)1 : (byte)0;	// search_direction = odczytany DIR

         // check for no devices on 1-Wire
         if ((id_bit) && (cmp_id_bit))
            break;
         else
         {
            if ((!id_bit) && (!cmp_id_bit) && (search_direction == 0))
            {
               last_zero = id_bit_number;

               // check for Last discrepancy in family
               if (last_zero < 9)
                  LastFamilyDiscrepancy = last_zero;
            }

            // set or clear the bit in the ROM byte rom_byte_number
            // with mask rom_byte_mask
            if (search_direction == 1)
               OW_SEARCH_ROM_NO[rom_byte_number] |= rom_byte_mask;
            else
               OW_SEARCH_ROM_NO[rom_byte_number] &= (byte)~rom_byte_mask;

            // increment the byte counter id_bit_number
            // and shift the mask rom_byte_mask
            id_bit_number++;
            rom_byte_mask <<= 1;

            // if the mask is 0 then go to new SerialNum byte rom_byte_number
            // and reset mask
            if (rom_byte_mask == 0)
            {
				_crc_ibutton_update( crc8, OW_SEARCH_ROM_NO[rom_byte_number]);
               rom_byte_number++;
               rom_byte_mask = 1;
            }
         }
      }
      while(rom_byte_number < 8);  // loop until through all ROM bytes 0-7

      // if the search was successful then
      if (!((id_bit_number < 65) || (crc8 != 0)))
      {
         // search successful so set LastDiscrepancy,LastDeviceFlag
         // search_result
         LastDiscrepancy = last_zero;

         // check for last device
         if (LastDiscrepancy == 0)
            LastDeviceFlag = TRUE;

         search_result = TRUE;
      }
   }

   // if no device found then reset counters so next
   // 'search' will be like a first

   if (!search_result || (OW_SEARCH_ROM_NO[0] == 0))
   {
      LastDiscrepancy = 0;
      LastDeviceFlag = FALSE;
      LastFamilyDiscrepancy = 0;
      search_result = FALSE;
   }

   return search_result;
}


//======================================================================//

